!TS PNETCDF
#include <misc.h>
#include <params.h>
#if ( defined SCAM )
#include <max.h>
#endif
module pnetcdfio
!----------------------------------------------------------------------- 
! 
! Purpose: pnetcdf IO module. Contains functions and subroutine for parallel IO
!
! Public functions/subroutines:
!   pnetcdf_output
! 
! Author: CCM Core Group
!------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8, r4 => shr_kind_r4
   implicit none
   private
   public pnetcdf_output,mype,comm_cart
   integer mype,comm_cart
   contains
   subroutine pnetcdf_output
      use mpishorthand,only:mpicom,mpir4,mpi_sum,mpi_min,mpi_max
      use spmd_dyn,only:npes
#  include "pnetcdf.inc"
!TS_PNETCDF
      integer NREADS,NWRITES
      logical reorder,isperiodic(3)
      integer ierr,totpes,numpes(3),pe_coords(3)
      NFMPI_OFFSET istart, jstart, kstart      ! offsets of 3D field
      NFMPI_OFFSET locsiz
      NFMPI_OFFSET TOTSIZ_3D(3),LOCSIZ_3D(3)
      real(r4) filsiz

      real(r4) rdt_g(2),rdt_l(2),wrt_g(2),wrt_l(2)
      real(r4) rrates_g(2),rrates_l(2),wrates_g(2),wrates_l(2)
      data numpes / 1, 0, 0 /
      data TOTSIZ_3D / 2, 3, 4 /
      NREADS=2;NWRITES=2

      call MPI_Dims_Create(npes, 3, numpes, ierr)
      call MPI_Cart_Create(mpicom, 3, numpes,&
	isperiodic, reorder,comm_cart, ierr)
      call MPI_Comm_Rank (mpicom, mype, ierr)
      call MPI_Cart_Coords (comm_cart, mype, 3, pe_coords, ierr)
      rdt_l(1) = 1.0e38
      rdt_l(2) = 1.0e38
      wrt_l(1) = 1.0e38
      wrt_l(2) = 1.0e38
      call Find_Locnx(TOTSIZ_3D(1),pe_coords(1),numpes(1),locsiz_3d(1),istart)
      call Find_Locnx(TOTSIZ_3D(2),pe_coords(2),numpes(2),locsiz_3d(2),jstart)
      call Find_Locnx(TOTSIZ_3D(3),pe_coords(3),numpes(3),locsiz_3d(3),kstart)
      filsiz = (TOTSIZ_3D(1) * TOTSIZ_3D(2) * TOTSIZ_3D(3))*1.0d-6 * 4.0
      if (mype == 0) Write (6,900)
      call MPI_Barrier (comm_cart, ierr)
      Write (6, 902)mype, pe_coords(1), pe_coords(2), pe_coords(3),TOTSIZ_3D(1), TOTSIZ_3D(2), TOTSIZ_3D(3),locsiz_3d(1), locsiz_3d(2), locsiz_3d(3),kstart, jstart, istart

 900  format ("mype  pe_coords    totsiz_3d         locsiz_3d       ","kstart,jstart,istart")
 902  format (i3,3x,i2,1x,i2,1x,i2,2x,i4,1x,i4,1x,i4,4x,i4,1x,i4,1x,i4,3x,i6,1x,i6,1x,i6)
!     -------------------------
!     Write and then read back.
!     -------------------------
      locsiz = locsiz_3d(1) * locsiz_3d(2) * locsiz_3d(3)
!     ===============
      call Write_File("pnf_test.nc", NWRITES, mype, comm_cart, istart, jstart,kstart, locsiz, locsiz_3d, TOTSIZ_3D, wrt_l)
!     ==============
      call Read_File("pnf_test.nc", NREADS,  mype, comm_cart, istart, jstart,kstart, locsiz, locsiz_3d, TOTSIZ_3D, rdt_l)

!     ----------------------------
!     Compute and print I/O rates.
!     ----------------------------
      wrates_l(1) = filsiz / wrt_l(2)               ! write rate
      wrates_l(2) = filsiz / (wrt_l(1) + wrt_l(2))  ! effective write rate
      rrates_l(1) = filsiz / rdt_l(2)               ! read rate
      rrates_l(2) = filsiz / (rdt_l(1) + rdt_l(2))  ! effective read  rate

      call MPI_Allreduce(wrates_l,wrates_g,2,mpir4,MPI_MIN,comm_cart,ierr)
      call MPI_Allreduce(rrates_l,rrates_g,2,mpir4,MPI_MIN,comm_cart,ierr)
      call MPI_Allreduce(wrt_l,wrt_g,2,mpir4,MPI_MAX,comm_cart,ierr)
      call MPI_Allreduce(rdt_l,rdt_g,2,mpir4,MPI_MAX,comm_cart,ierr)

      if (mype == 0) then
        Write (6,905) filsiz
        Write (6,910) wrates_g(1), wrates_g(2)
        Write (6,915) rrates_g(1), rrates_g(2)
        Write (6,920) totpes
        Write(6,922)wrt_g(1),wrt_g(2),wrates_g(2),rdt_g(1),rdt_g(2),rrates_g(2)
      end if 
 905  format ("File size: ", e10.3, " MB")
 910  format ("    Write: ", f9.3, " MB/s  (eff., ", f9.3, " MB/s)")
 915  format ("    Read : ", f9.3, " MB/s  (eff., ", f9.3, " MB/s)")
 920  format ("Total number PEs: ", i4)
 922  format (e11.3, e11.3, f9.3, e11.3, e11.3, f9.3)

      end subroutine pnetcdf_output
!TS_PNETCDFE
      subroutine Write_File(filename, nwrites, mype, comm_cart, istart, jstart, kstart,locsiz, locsiz_3d, totsiz_3d, wrt_l)
      implicit none
      include "mpif.h"
#     include "pnetcdf.inc"
!     ----------------------
!     Argument declarations.
!     ----------------------
      character (len=*) filename
      integer nwrites
      integer mype
      integer comm_cart
      NFMPI_OFFSET istart, jstart, kstart
      NFMPI_OFFSET locsiz
      NFMPI_OFFSET locsiz_3d(3)
      NFMPI_OFFSET totsiz_3d(3)
      real(r4)  wrt_l(2)


!     ----------------------------
!     Local variable declarations.
!     ----------------------------

      integer ierr
      integer lon_id, lat_id, lev_id,time_id
      integer ncid,nw,tt_id
              
      NFMPI_OFFSET count_3d(3)
      NFMPI_OFFSET start_3d(3)
              
      integer dim_id(3)
      double precision  t1, t2, t3
      integer max_loc_size
      parameter( max_loc_size = 20000000 )
      real(r4)  tt(max_loc_size)   ! Need tt(locsiz)
      real(r4) ps0
      if (locsiz .gt. MAX_LOC_SIZE) then
         print *, 'locsiz = ', locsiz, ' larger than MAX_LOC_SIZE'
         stop
      endif
!     ----------------
!     Begin execution.
!     ----------------
      start_3d(1) = istart
      start_3d(2) = jstart
      start_3d(3) = kstart
      count_3d(1) = locsiz_3d(1)
      count_3d(2) = locsiz_3d(2)
      count_3d(3) = locsiz_3d(3)

      do nw = 1, nwrites
        call Get_Field(istart,jstart,kstart,locsiz,locsiz_3d,totsiz_3d,tt)
        call MPI_Barrier (comm_cart, ierr)
        t1 = MPI_Wtime ( )
        ierr = Nfmpi_Create(comm_cart,filename,NF_CLOBBER,MPI_INFO_NULL,ncid)

        ierr = Nfmpi_Def_Dim(ncid, "level",     totsiz_3d(1), lon_id)
        ierr = Nfmpi_Def_Dim(ncid, "latitude",  totsiz_3d(2), lat_id)
        ierr = Nfmpi_Def_Dim(ncid, "longitude", totsiz_3d(3), lev_id)
        ierr = Nfmpi_Def_Dim(ncid, "time", nf_unlimited, time_id)

        dim_id(1) = lon_id
        dim_id(2) = lat_id
        dim_id(3) = lev_id
	ps0=8.
        ierr = Nfmpi_Def_Var(ncid, "tt", NF_REAL, 3, dim_id, tt_id)
!TS        ierr = Nfmpi_Def_Var(ncid, "tt", NF_REAL, 0, 0, tt_id)
        ierr = Nfmpi_Enddef (ncid)
        t2 = MPI_Wtime ( )
	print*,'start3d',start_3d,count_3d
	print*,'tt',tt(1)
        ierr = Nfmpi_Put_Vara_Real_All(ncid, tt_id, start_3d, count_3d, tt)
!TS	print*,'pnetcdf-def',ncid,tt_id,ps0,ps0
!TS  	ierr = nfmpi_put_var_real (ncid, tt_id, tt(2))
!TS  	ierr = nfmpi_put_var_double (ncid, tt_id, ps0)
	print*,'pnetcdf-double',ierr
        ierr = Nfmpi_Close (ncid)
	print*,'pnetcdf-close',ierr
        call MPI_Barrier (comm_cart, ierr)
        t3 = MPI_Wtime ( )
        if (t2 - t1 < wrt_l(1)) wrt_l(1) = t2 - t1
        if (t3 - t2 < wrt_l(2)) wrt_l(2) = t3 - t2
        if (mype == 0) Write (6,950) nw, t2-t1, t3-t2
      end do
 950  format ("write ", i1, ": ", e9.3, 1x, e9.3)
      Return
      end subroutine Write_File

!     -----------
      subroutine Read_File(filename, nreads, mype, comm_cart, istart, jstart, kstart,locsiz, locsiz_3d, totsiz_3d, rdt_l)
      implicit none
      include "mpif.h"
#     include "pnetcdf.inc"
!     ----------------------
!     Argument declarations.
!     ----------------------
      character (len=*) filename
      integer nreads
      integer mype
      integer comm_cart
      NFMPI_OFFSET istart, jstart, kstart
      NFMPI_OFFSET locsiz
      NFMPI_OFFSET locsiz_3d(3)
      NFMPI_OFFSET totsiz_3d(3)
      real(r4)  rdt_l(2)
!     ----------------------------
!     Local variable declarations.
!     ----------------------------
      integer ierr,ncid,nr,tt_id,ii
      NFMPI_OFFSET count_3d(3)
      NFMPI_OFFSET start_3d(3)
      double precision  t1, t2, t3
      integer max_loc_size
      parameter( max_loc_size = 20000000 )
      real(r4)  buf(max_loc_size)
      real(r4)  tt (max_loc_size)
      NFMPI_OFFSET dimlen

!     ----------------
!     Begin execution.
!     ----------------
      call Get_Field(istart, jstart, kstart, locsiz, locsiz_3d, totsiz_3d, tt)

!     start_3d(1:3) = (/ kstart, jstart, istart /)
!     count_3d(:)   = locsiz_3d(:)
      start_3d(1) = istart
      start_3d(2) = jstart
      start_3d(3) = kstart
      count_3d(1) = locsiz_3d(1)
      count_3d(2) = locsiz_3d(2)
      count_3d(3) = locsiz_3d(3)

      do nr = 1, nreads
         do ii=1,locsiz
            buf(ii) =  -99.99
         enddo

        call MPI_Barrier (comm_cart, ierr)
        t1 = MPI_Wtime ( )
        ierr = Nfmpi_Open(comm_cart, filename,NF_NOWRITE, MPI_INFO_NULL, ncid)
        ierr = Nfmpi_Inq_Varid(ncid, "tt", tt_id)
        ierr = nfmpi_inq_dimlen (ncid, tt_id, dimlen)
	print*,'dimlen',dimlen
        t2 = MPI_Wtime ( )
        ierr = Nfmpi_Get_Vara_Real_All(ncid, tt_id, start_3d, count_3d, buf)
!       ================
        ierr = Nfmpi_Close (ncid)
!       ================
        call MPI_Barrier (comm_cart, ierr)
        t3 = MPI_Wtime ( )
        if (t2 - t1 < rdt_l(1)) rdt_l(1) = t2 - t1
        if (t3 - t2 < rdt_l(2)) rdt_l(2) = t3 - t2
        if (mype == 0) Write (6,970) nr, t2-t1, t3-t2
        if (nr == 1)call Compare_Vec(mype, comm_cart, locsiz, tt, buf)
      end do
 970  format (" read ", i1, ": ", e9.3, 1x, e9.3)
      Return
      end subroutine Read_File
!     ------------
      subroutine Find_Locnx(nx, mype, totpes, locnx, ibegin)
      implicit none
      include "mpif.h"
#     include "pnetcdf.inc"
!     ----------------------
!     Argument declarations.
!     ----------------------
      NFMPI_OFFSET nx
      integer mype,totpes
      NFMPI_OFFSET locnx
      NFMPI_OFFSET ibegin
!     ----------------------------
!     Local variable declarations.
!     ----------------------------
      NFMPI_OFFSET iremain
!     ----------------
!     Begin execution.
!     ----------------
      locnx = nx / totpes
      iremain = nx - (totpes * locnx)
      if (mype < iremain) locnx = locnx + 1
      ibegin = mype * (nx / totpes) + iremain + 1
      if (mype < iremain) ibegin = ibegin + (mype - iremain)
      Return
      end subroutine Find_Locnx
!     ------------
      subroutine Get_Field(istart,jstart,kstart,locsiz,locsiz_3d,totsiz_3d,tt)
      implicit none
      include "mpif.h"
#     include "pnetcdf.inc"
!     ----------------------
!     Argument declarations.
!     ----------------------
      NFMPI_OFFSET istart, jstart, kstart
      NFMPI_OFFSET locsiz
      NFMPI_OFFSET locsiz_3d(3)
      NFMPI_OFFSET totsiz_3d(3)
      real(r4)  tt(locsiz)
!     ----------------------------
!     Local variable declarations.
!     ----------------------------
      integer ii, jj, kk,ind
!     ----------------
!     Begin execution.
!     ----------------
      ind = 1
      do kk = 1, locsiz_3d(3)
        do jj = 1, locsiz_3d(2)
          do ii = 1, locsiz_3d(1)
             tt(ind)=(istart-1 +(ii - 1) + 1 + totsiz_3d(3)*(jstart-1 +&
              (jj - 1) + totsiz_3d(2)*(kstart-1 +(kk-1)))) * 1.0d-3
             ind = ind + 1
          end do
        end do
      end do
      Return
      end subroutine Get_Field

!     ------------

      subroutine Compare_Vec(mype, comm_cart, locsiz, tt, buf)
      use mpishorthand
      implicit none
!TS      include "mpif.h"
!     ----------------------
!     Argument declarations.
!     ----------------------
      integer mype
      integer comm_cart
      NFMPI_OFFSET locsiz
      real(r4)  tt (locsiz)
      real(r4)  buf(locsiz)
!     ----------------------------
!     Local variable declarations.
!     ----------------------------
      integer ierr
      integer ii              
      real(r4)  delmax, delmin, delta
      real(r4)  diff              
      real(r4)  wr(5)
      real(r4)  ws(5)
!     ----------------
!     Begin execution.
!     ----------------
      ws(1) = 0.0d0      ! diff
      ws(2) = 0.0d0      ! sumsq
      ws(3) = locsiz     ! locsiz
      ws(4) = 0.0d0      ! delmax
      ws(5) = 1.0d38     ! Huge (ws)  ! delmin
      do ii = 1, locsiz
        delta = (tt(ii) - buf(ii)) * (tt(ii) - buf(ii))
        ws(1) = ws(1) + delta
        ws(2) = ws(2) + tt(ii) * tt(ii)
        if (delta > ws(4)) ws(4) = delta
        if (delta < ws(5)) ws(5) = delta
      end do
      call MPI_Allreduce(ws,wr,3,mpir4,MPI_SUM,comm_cart,ierr)
      call MPI_Allreduce(ws(4),delmax,1,mpir4,MPI_MAX,comm_cart,ierr)
      call MPI_Allreduce(ws(5),delmin,1,mpir4,MPI_MIN,comm_cart,ierr)

      diff   = Sqrt (wr(1) / wr(2))         ! normalized error
      delmax = Sqrt (wr(3) * delmax/wr(2))  ! normalized max difference
      delmin = Sqrt (wr(3) * delmin/wr(2))  ! normalized min difference

      if (mype == 0) Write (6,990) diff, delmax, delmin
 990  format ("diff, delmax, delmin = ",e9.3, 1x, e9.3, 1x, e9.3)
      Return

      end subroutine Compare_Vec
end module pnetcdfio
