#include <misc.h>
#include <params.h>
#include <preproc.h>

!-------------------------------------------------------------------------------
!
! Purpose:
!
! Wrapper routines for the netCDF library for input and output data.
!
! Author: Jim Rosinski
!
! $Id: wrap_nf.F90,v 1.4.10.1 2002/06/15 13:50:09 erik Exp $
!
!-------------------------------------------------------------------------------
!===============================================================================

   subroutine pwrap_create (path, cmode, ncid)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Create a netCDF file for reading and/or writing
!
!------------------------------------------------------------------------------- 
   use pnetcdfio,only:comm_cart
   implicit none
   include "mpif.h"
#  include "pnetcdf.inc"
   character*(*), intent(in):: path
   integer, intent(in):: cmode
   integer, intent(out):: ncid
   integer ret      ! NetCDF return code

   ret = nfmpi_create (comm_cart,path, cmode,MPI_INFO_NULL,ncid)
   if (ret/=0) call pzio_handle_error (ret)
   end subroutine pwrap_create

!===============================================================================
   subroutine pwrap_inq_dim (nfid, dimid, dimname, dimlen)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Gets dimension name for a given dimension id
!
!-------------------------------------------------------------------------------
   implicit none
#  include "pnetcdf.inc"
   integer, intent(in)::  nfid
   integer, intent(in):: dimid
   integer, intent(out):: dimlen
   character*(*), intent(out):: dimname

   integer ret      ! NetCDF return code

   ret = nfmpi_inq_dim (nfid, dimid, dimname,dimlen)
   if (ret/=0) call pzio_handle_error (ret)
   end subroutine pwrap_inq_dim

!===============================================================================

   subroutine pwrap_inq_dimid (nfid, dimname, dimid)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Gets the dimension id
!
!-------------------------------------------------------------------------------
   implicit none
#  include "pnetcdf.inc"
   integer, intent(in):: nfid
   integer, intent(out):: dimid
   character*(*), intent(in):: dimname

   integer ret      ! NetCDF return code
!TS   print*,'in pwrap_inq_dimid',nfid,dimname,dimid
   ret = nfmpi_inq_dimid (nfid, dimname, dimid)
!TS   print*,'in pwrap_inq_dimid',dimid,ret
   if (ret/=0) call pzio_handle_error (ret)
   end subroutine pwrap_inq_dimid

!===============================================================================

   subroutine pwrap_inq_dimlen (nfid, dimid, dimlen)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Gets the dimension length for a given dimension
!
!-------------------------------------------------------------------------------
   implicit none
#  include "pnetcdf.inc"
   integer, intent(in)::  nfid
   integer, intent(in)::  dimid 
   integer, intent(out):: dimlen
   NFMPI_OFFSET len
   
   integer ret      ! NetCDF return code

   ret = nfmpi_inq_dimlen (nfid, dimid, len)
   dimlen=len
   if (ret/=0) call pzio_handle_error (ret)
   end subroutine pwrap_inq_dimlen

!===============================================================================

!TS   subroutine pwrap_inq_nvars (nfid, nvars)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Returns the number of variables
!
!-------------------------------------------------------------------------------
!TS   use zio
!TS   implicit none
!TS   integer, intent(in):: nfid
!TS   integer, intent(out):: nvars
!TS   integer ret      ! NetCDF return code

!TS   ret = zio_nf_inq_nvars (nfid, nvars)
!TS   if (ret/=ZIO_NOERR) call pzio_handle_error (ret)
!TS   end subroutine pwrap_inq_nvars

!===============================================================================

   subroutine pwrap_inq_vardimid (nfid, varid, dimids)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Returns the dimension Id's from a variable
!
!-------------------------------------------------------------------------------
   implicit none
#  include "pnetcdf.inc"
   integer, intent(in):: nfid
   integer, intent(in):: varid
   integer, intent(out)::  dimids(*)
   
   integer ret      ! NetCDF return code

   ret = nfmpi_inq_vardimid (nfid, varid, dimids)
   if (ret/=0) call pzio_handle_error (ret)
   end subroutine pwrap_inq_vardimid

!===============================================================================

   subroutine pwrap_inq_varid (nfid, varname, varid)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Returns the variable ID
!
!-------------------------------------------------------------------------------
   implicit none
#  include "pnetcdf.inc"
   integer, intent(in):: nfid
   integer, intent(out):: varid
   character*(*), intent(in):: varname
   
   integer ret      ! NetCDF return code

   ret = nfmpi_inq_varid (nfid, varname, varid)
   if (ret/=0) then
     write(6,*)'wrap_inq_varid: id for ',trim(varname),' not found'
     call pzio_handle_error (ret)
   end if
   end subroutine pwrap_inq_varid

!===============================================================================

   subroutine pwrap_inq_var (nfid, varid, varname, xtype, ndims, &
                            dimids, natts)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Returns the variable name, type, number of dimensions, dimension ID's, and number of attributes
!
!-------------------------------------------------------------------------------
   implicit none
#  include "pnetcdf.inc"
   integer, intent(in):: nfid
   integer, intent(in):: varid
   integer, intent(out):: xtype
   integer, intent(out):: ndims
   integer, intent(out):: dimids(*)
   integer, intent(out):: natts
   character*(*), intent(out):: varname

   integer ret      ! NetCDF return code

   ret = nfmpi_inq_var (nfid, varid, varname, xtype, ndims, dimids, &
                         natts)
   if (ret/=0) call pzio_handle_error (ret)
   end subroutine pwrap_inq_var

!===============================================================================

!TS   subroutine pwrap_inq_varname (nfid, varid, varname)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Returns the variable name from the dimension ID
!
!-------------------------------------------------------------------------------
!TS   use zio
!TS   implicit none
!TS   integer ret      ! NetCDF return code

!TS   integer, intent(in):: nfid
!TS   integer, intent(in):: varid
!TS   character*(*), intent(out):: varname

!TS   ret = zio_nf_inq_varname (nfid, varid, varname)
!TS   if (ret/=ZIO_NOERR) call pzio_handle_error (ret)
!TS   end subroutine pwrap_inq_varname

!===============================================================================

   subroutine pwrap_get_att_text (nfid, varid, attname, atttext)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Returns the attribute text from the given variable ID and attribute name
!
!-------------------------------------------------------------------------------
   implicit none
#  include "pnetcdf.inc"
   integer, intent(in):: nfid
   integer, intent(in):: varid
   character*(*), intent(in):: attname
   character*(*), intent(out):: atttext

   integer ret      ! NetCDF return code

   ret = nfmpi_get_att_text (nfid, varid, attname, atttext)
   if (ret/=0) call pzio_handle_error (ret)
   end subroutine pwrap_get_att_text

!===============================================================================

   subroutine pwrap_put_att_text (nfid, varid, attname, atttext)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Puts the given attribute text to variable ID.
!
! This routine violates the convetion that the wrapper codes take an identical
! set of arguments as the netcdf library code.  The length of the character
! argument is computed inside the wrapper.
!
!-------------------------------------------------------------------------------

   implicit none
#  include "pnetcdf.inc"
   integer, intent(in):: nfid
   integer, intent(in):: varid
   character*(*), intent(in):: attname
   character*(*), intent(in):: atttext

   integer ret      ! NetCDF return code
   NFMPI_OFFSET siz

   siz = len_trim(atttext)
   ret = nfmpi_put_att_text (nfid, varid, attname, siz, atttext)
   if (ret/=0) call pzio_handle_error (ret)
   end subroutine pwrap_put_att_text

!===============================================================================

   subroutine pwrap_put_att_realx (nfid, varid, attname, xtype, len, &
                                  attval)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Puts the given real attribute to the variable id
!
!-------------------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   implicit none
#  include "pnetcdf.inc"
   integer , intent(in):: nfid
   integer , intent(in):: varid
   integer , intent(in):: xtype
!TS   integer , intent(in):: len
   character*(*) , intent(in):: attname
   real(r8), intent(in):: attval(*)

   integer ret      ! NetCDF return code
   NFMPI_OFFSET len
#ifdef CRAY
   ret = nfmpi_put_att_real (nfid, varid, attname, xtype, len, attval)
#else
   ret = nfmpi_put_att_double (nfid, varid, attname, xtype, len, attval)
#endif
   if (ret/=0) call pzio_handle_error (ret)
   end subroutine pwrap_put_att_realx
!===============================================================================

!TS   subroutine pwrap_def_dim (nfid, dimname, len, dimid)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Defines the input dimension
!
!-------------------------------------------------------------------------------
!TS   use zio
!TS   implicit none
!TS   integer, intent(in):: nfid
!TS   integer, intent(in):: len
!TS   integer, intent(out):: dimid
!TS   character*(*), intent(in):: dimname
   
!TS   integer ret      ! NetCDF return code

!TS   ret = zio_nf_def_dim (nfid, dimname, len, dimid)
!TS   if (ret/=ZIO_NOERR) call pzio_handle_error (ret)
!TS   end subroutine pwrap_def_dim

!===============================================================================

   subroutine pwrap_def_var (nfid, name, xtype, nvdims, vdims, varid)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Defines the given variable
!
!-------------------------------------------------------------------------------
   implicit none
#  include "pnetcdf.inc"
   integer, intent(in):: nfid
   integer, intent(in)::xtype
   integer, intent(in)::nvdims
   integer, intent(out)::varid
   integer, intent(in):: vdims(nvdims+1)
   character*(*), intent(in):: name
   
   integer ret      ! NetCDF return code

   ret = nfmpi_def_var (nfid, name, xtype, nvdims, vdims, varid)
   if (ret/=0) call pzio_handle_error (ret)
   end subroutine pwrap_def_var

!===============================================================================

   subroutine pwrap_get_var_realx (nfid, varid, arr)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Gets the given real variable from a input file
!
!-------------------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   implicit none
#  include "pnetcdf.inc"
   integer, intent(in):: nfid
   integer, intent(in):: varid
   real(r8), intent(out):: arr(*)

   integer ret      ! NetCDF return code

#ifdef CRAY
   ret = nfmpi_get_var_real_all (nfid, varid, arr)
#else
   ret = nfmpi_get_var_double_all (nfid, varid, arr)
#endif
   if (ret/=0) then
     write(6,*)'WRAP_GET_VAR_REALX: error reading varid =', varid
     call pzio_handle_error (ret)
   end if
   end subroutine pwrap_get_var_realx

!===============================================================================

   subroutine pwrap_get_var_int (nfid, varid, arr)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Gets the given integer variable from a input file
!
!-------------------------------------------------------------------------------
   implicit none
#  include "pnetcdf.inc"
   integer, intent(in):: nfid
   integer, intent(in):: varid
   integer, intent(out):: arr(*)

   integer ret      ! NetCDF return code

   ret = nfmpi_get_var_int_all (nfid, varid, arr)
   if (ret/=0) then
     write(6,*)'WRAP_GET_VAR_INT: error reading varid =', varid
     call pzio_handle_error (ret)
   end if
   end subroutine pwrap_get_var_int

!===============================================================================

   subroutine pwrap_get_vara_realx (nfid, varid, start, count, arr)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Gets a range of the given real variable from a input file
!
!-------------------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   implicit none
#  include "pnetcdf.inc"
   integer, intent(in):: nfid
   integer, intent(in)::varid
   NFMPI_OFFSET, intent(in)::start(*)
   NFMPI_OFFSET, intent(in)::count(*)
   real(r8), intent(out):: arr(*)

   integer ret      ! NetCDF return code

#ifdef CRAY
   ret = nfmpi_get_vara_real_all (nfid, varid, start, count, arr)
#else
   ret = nfmpi_get_vara_double_all (nfid, varid, start, count, arr)
#endif
   if (ret/=0) then
     write(6,*)'WRAP_GET_VARA_REALX: error reading varid =', varid
     call pzio_handle_error (ret)
   end if
   end subroutine pwrap_get_vara_realx

!===============================================================================

   subroutine pwrap_get_vara_int (nfid, varid, start, count, arr)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Gets a range of the given integer variable from a input file.
!
!-------------------------------------------------------------------------------
   implicit none
#  include "pnetcdf.inc"
   integer, intent(in):: nfid
   integer, intent(in):: varid
   NFMPI_OFFSET, intent(in):: start(*)
   NFMPI_OFFSET, intent(in):: count(*)
   integer, intent(out):: arr(*)

   integer ret      ! NetCDF return code

   ret = nfmpi_get_vara_int_all (nfid, varid, start, count, arr)
   if (ret/=0) then
     write(6,*)'WRAP_GET_VARA_INT: error reading varid =', varid
     call pzio_handle_error (ret)
   end if
   end subroutine pwrap_get_vara_int

!===============================================================================

   subroutine pwrap_open (path, omode, ncid)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Open a netCDF file
!
!-------------------------------------------------------------------------------
   use pnetcdfio,only:comm_cart
   implicit none
   include "mpif.h"
#  include "pnetcdf.inc"
   character*(*), intent(in):: path
   integer, intent(in):: omode
   integer, intent(out):: ncid

   integer ret      ! NetCDF return code

   ret = nfmpi_open (comm_cart,path, omode,MPI_INFO_NULL, ncid)
   if (ret/=0) then
     write(6,*)'PWRAP_OPEN: nf_open failed for file ',path
     call pzio_handle_error (ret)
   end if
   end subroutine pwrap_open

!===============================================================================

   subroutine pwrap_close (ncid)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Close netCDF file
!
!-------------------------------------------------------------------------------
   implicit none
   include "mpif.h"
#  include "pnetcdf.inc"
   integer, intent(in):: ncid

   integer ret      ! NetCDF return code

   ret = nfmpi_close (ncid)
   if (ret/=0) then
     write(6,*)'PWRAP_CLOSE: nf_close failed for id ',ncid
     call pzio_handle_error (ret)
   end if
   end subroutine pwrap_close

!===============================================================================

   subroutine pwrap_put_var_int (nfid, varid, arr)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Put a integer variable on output file.
!
!-------------------------------------------------------------------------------
   implicit none
#  include "pnetcdf.inc"
   integer, intent(in):: nfid
   integer, intent(in):: varid
   integer, intent(in):: arr(*)

   integer ret      ! NetCDF return code

   ret = nfmpi_put_var_int (nfid, varid, arr)
   if (ret/=0) call pzio_handle_error (ret)
   end subroutine pwrap_put_var_int

!===============================================================================

!TS   subroutine pwrap_put_var1_int (nfid, varid, index, ival)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Put a variable on output file at a given index.
!
!-------------------------------------------------------------------------------
!TS   use zio
!TS   implicit none
!TS   integer, intent(in):: nfid
!TS   integer, intent(in):: varid
!TS   integer, intent(in):: index(*)
!TS   integer, intent(in):: ival

!TS   integer ret      ! NetCDF return code

!TS   ret = zio_nf_put_var1_int (nfid, varid, index, ival)
!TS   if (ret/=ZIO_NOERR) call pzio_handle_error (ret)
!TS   end subroutine pwrap_put_var1_int

!===============================================================================

   subroutine pwrap_put_vara_int (nfid, varid, start, count, arr)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Put a range of a integer variable on a output file.
!
!-------------------------------------------------------------------------------
   implicit none
#  include "pnetcdf.inc"
   integer, intent(in):: nfid
   integer, intent(in):: varid
!TS   integer, intent(in):: start(*)
!TS   integer, intent(in):: count(*)
   NFMPI_OFFSET, intent(in):: start(*)
   NFMPI_OFFSET, intent(in):: count(*)
   integer, intent(in):: arr(*)
   integer ret      ! NetCDF return code

   ret = nfmpi_put_vara_int_all (nfid, varid, start, count, arr)
!TS   print*,'put_vara_int',ret,start(1),count(1),arr(1),nfid,varid
   if (ret/=0) call pzio_handle_error (ret)
   end subroutine pwrap_put_vara_int

!===============================================================================

   subroutine pwrap_put_vara_text (nfid, varid, start, count, text)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Put a range of the given text variable to output file.
!
!-------------------------------------------------------------------------------
   implicit none
#  include "pnetcdf.inc"
   integer, intent(in):: nfid
   integer, intent(in):: varid
   NFMPI_OFFSET, intent(in):: start(*)
   NFMPI_OFFSET, intent(in):: count(*)
   character*(*), intent(in):: text

   integer ret      ! NetCDF return code

   ret = nfmpi_put_vara_text_all (nfid, varid, start, count, text)
   if (ret/=0) call pzio_handle_error (ret)
   end subroutine pwrap_put_vara_text

!===============================================================================

   subroutine pwrap_put_var1_realx (nfid, varid, index, val)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Put the given real variable to output file at given index.
!
!-------------------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   implicit none
#  include "pnetcdf.inc"
   integer, intent(in):: nfid
   integer, intent(in):: varid
   NFMPI_OFFSET, intent(in):: index(*)
   real(r8), intent(in):: val

   integer ret      ! NetCDF return code

#ifdef CRAY
   ret = nfmpi_put_var1_real (nfid, varid, index, val)
#else
   ret = nfmpi_put_var1_double (nfid, varid, index, val)
#endif
   if (ret/=0) call pzio_handle_error (ret)
   end subroutine pwrap_put_var1_realx

!===============================================================================

   subroutine pwrap_put_vara_realx (nfid, varid, start, count, arr)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Output the given portion of the real array.
!
!-------------------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   implicit none
#  include "pnetcdf.inc"
   integer, intent(in):: nfid
   integer, intent(in):: varid
   NFMPI_OFFSET, intent(in):: start(*)
   NFMPI_OFFSET, intent(in):: count(*)
   real(r8), intent(in):: arr(*)
   integer ret      ! NetCDF return code

#ifdef CRAY
   ret = nfmpi_put_vara_real_all (nfid, varid, start, count, arr)
#else
   ret = nfmpi_put_vara_double_all (nfid, varid, start, count, arr)
!TS   print*,'put_vara_real',ret,start(1),count(1),nfid,varid
#endif
   if (ret/=0) call pzio_handle_error (ret)
   end subroutine pwrap_put_vara_realx
!===============================================================================
   subroutine pwrap_put_vara_realx_us (nfid, varid, start, count, arr,distarray)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Output the given portion of the real array.
!
!-------------------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   implicit none
#  include "pnetcdf.inc"
   integer, intent(in):: nfid
   integer, intent(in):: varid
   NFMPI_OFFSET, intent(in):: start(*)
   NFMPI_OFFSET, intent(in):: count(*)
   real(r8), intent(in):: arr(*)
   integer ret      ! NetCDF return code
   integer, intent(in):: distarray

#ifdef CRAY
   ret = nfmpi_put_vara_all (nfid, varid, start, count, arr,1,distarray)
#else
!TS   print*,'put_vara_real',start(1:3),count(1:3),nfid,varid,distarray
   ret = nfmpi_put_vara_all (nfid, varid, start, count, arr,1,distarray)
!TS   print*,'put_vara_real',ret,start(1),count(1),nfid,varid
#endif
   if (ret/=0) call pzio_handle_error (ret)
   end subroutine pwrap_put_vara_realx_us
!===============================================================================
   subroutine pwrap_put_vara_real (nfid, varid, start, count, arr)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Output the given portion of the real array.
!
!-------------------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8, r4 => shr_kind_r4
   implicit none
#  include "pnetcdf.inc"
   integer, intent(in):: nfid
   integer, intent(in):: varid
   NFMPI_OFFSET, intent(in):: start(*)
   NFMPI_OFFSET, intent(in):: count(*)
   real(r4), intent(in):: arr(*)

   integer ret      ! NetCDF return code
   ret = nfmpi_put_vara_real_all (nfid, varid, start, count, arr)
   if (ret/=0) call pzio_handle_error (ret)
   end subroutine pwrap_put_vara_real
!===============================================================================
   subroutine pwrap_put_vara_real_us (nfid, varid, start, count, arr,distarray)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Output the given portion of the real array.
!
!-------------------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8, r4 => shr_kind_r4
   implicit none
#  include "pnetcdf.inc"
   integer, intent(in):: nfid
   integer, intent(in):: varid
   NFMPI_OFFSET, intent(in):: start(*)
   NFMPI_OFFSET, intent(in):: count(*)
   real(r4), intent(in):: arr(*)
   integer, intent(in):: distarray

   integer ret      ! NetCDF return code
   ret = nfmpi_put_vara_all (nfid, varid, start, count, arr,1,distarray)
   if (ret/=0) call pzio_handle_error (ret)
   end subroutine pwrap_put_vara_real_us

!===============================================================================

!TS   subroutine pwrap_put_var_real (nfid, varid, arr,distarray)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Put the given real variable to output file.
!
!-------------------------------------------------------------------------------
!TS   use shr_kind_mod, only: r4 => shr_kind_r4
!TS   use zio
!TS   implicit none
!TS   integer, intent(in):: nfid
!TS   integer, intent(in):: varid
!TS   real(r4), intent(in):: arr(*)
!TS   integer, intent(in):: distarray
!TS   integer ret      ! NetCDF return code

!TS   ret = zio_nf_put_var_real (nfid, varid, arr, distarray)
!TS   if (ret/=ZIO_NOERR) call pzio_handle_error (ret)
!TS   end subroutine pwrap_put_var_real

!===============================================================================

   subroutine pwrap_put_var_realx (nfid, varid, arr)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Put the given real variable to output file.
!
!-------------------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   implicit none
#  include "pnetcdf.inc"
   integer, intent(in):: nfid
   integer, intent(in):: varid
   real(r8), intent(in):: arr(*)
   real*4 arr_test
   integer ret      ! NetCDF return code
#ifdef CRAY
   ret = nfmpi_put_var_real (nfid, varid, arr)
#else
   ret = nfmpi_put_var_double (nfid, varid, arr)
!TS	print*,'wrap_put_var1',ret,arr(1),arr_test
#endif
   if (ret/=0) call pzio_handle_error (ret)
   end subroutine pwrap_put_var_realx

!===============================================================================
!   subroutine pwrap_set_fill (nfid, fillmode, old_mode)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Set the fill mode.
! This is not implemented in parallel netcdf
!
!-------------------------------------------------------------------------------
!===============================================================================

!TS   subroutine pwrap_sync (nfid)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Defines the input dimension
!
!-------------------------------------------------------------------------------
!TS   use zio
!TS   implicit none
!TS   integer, intent(in):: nfid
!TS   integer ret      ! NetCDF return code

!TS   ret = zio_nf_sync (nfid)
!TS   if (ret/=ZIO_NOERR) call pzio_handle_error (ret)

!TS   end subroutine pwrap_sync

!===============================================================================

   subroutine pzio_handle_error(ret)
!-------------------------------------------------------------------------------
!
! Purpose:
!
! Handle netCDF errors.
!
!-------------------------------------------------------------------------------
   use abortutils, only : endrun
   implicit none
#  include "pnetcdf.inc"
   integer, intent(in):: ret
   
   write(6,*)nfmpi_strerror(ret)
   call endrun('PNETCDF_HANDLE_ERROR')
   end subroutine pzio_handle_error

!===============================================================================

!TS   subroutine pzwrtout_r8 (filedesc, arr, distarray)
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Wrapper routine to write restart binary file 
! 
! Author: 
! 
!-----------------------------------------------------------------------
!TS   use shr_kind_mod, only: r8 => shr_kind_r8
!TS   use zio,          only: zio_uf_write_double, zio_get_fileid, zio_noerr
!TS   use abortutils, only : endrun
!
! Arguments
!
!TS   integer, intent(in) :: filedesc   ! File descriptor
!TS   integer, intent(in) :: distarray  ! Dist array descriptor
!TS   real(r8) :: arr(*)                ! Array to be written
!
! Local workspace
!
!TS   integer ioerr, ioerr1      ! errorcodes
!TS   integer iu

!TS   ioerr = zio_uf_write_double (filedesc, distarray, arr)

!TS   if (ioerr /= ZIO_NOERR) then
!TS      ioerr1 = zio_get_fileid (filedesc,iu)
!TS      write (6,*) 'WRTOUT_R8 ioerror ',ioerr,' on i/o unit = ',iu
!TS      call endrun('ZIO_ERROR')
!TS   end if

!TS   return
!TS   end subroutine pzwrtout_r8

!#######################################################################

!TS   subroutine pwrtout_r4 (filedesc, arr, distarray)
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Wrapper routine to write restart binary file 
! 
! Author: 
! 
!-----------------------------------------------------------------------
!TS   use shr_kind_mod, only: r4 => shr_kind_r4
!TS   use zio,          only: zio_uf_write_real, zio_get_fileid, zio_noerr
!TS   use abortutils, only : endrun
!
! Arguments
!
!TS   integer, intent(in) :: filedesc   ! File descriptor
!TS   integer, intent(in) :: distarray  ! Dist array descriptor
!TS   real(r4) :: arr(*)                ! Array to be written
!
! Local workspace
!
!TS   integer ioerr, ioerr1      ! errorcodes
!TS   integer iu

!TS   ioerr = zio_uf_write_real (filedesc, distarray, arr)

!TS   if (ioerr /= ZIO_NOERR) then
!TS      ioerr1 = zio_get_fileid (filedesc,iu)
!TS      write (6,*) 'WRTOUT_R4 ioerror ',ioerr,' on i/o unit = ',iu
!TS      call endrun('ZIO_ERROR')
!TS   end if

!TS   return
!TS   end subroutine pwrtout_r4

!#######################################################################

!TS   subroutine pwrtout_int (filedesc, arr, distarray)
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Wrapper routine to write restart binary file 
! 
! Author: 
! 
!-----------------------------------------------------------------------
!TS   use zio,          only: zio_uf_write_int, zio_get_fileid, zio_noerr
!TS   use abortutils, only : endrun
!
! Arguments
!
!TS   integer, intent(in) :: filedesc   ! File descriptor
!TS   integer, intent(in) :: distarray  ! Dist array descriptor
!TS   integer :: arr(*)                 ! Array to be written
!
! Local workspace
!
!TS   integer ioerr, ioerr1      ! errorcodes
!TS   integer iu

!TS   ioerr = zio_uf_write_int (filedesc, distarray, arr)

!TS   if (ioerr /= ZIO_NOERR) then
!TS      ioerr1 = zio_get_fileid (filedesc,iu)
!TS      write (6,*) 'WRTOUT_INT ioerror ',ioerr,' on i/o unit = ',iu
!TS      call endrun('ZIO_ERROR')
!TS   end if

!TS   return
!TS   end subroutine pwrtout_int

!#######################################################################

!TS   subroutine pzreadin_r8 (filedesc, arr, distarray)
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Wrapper routine to read binary file 
! 
! Author: 
! 
!-----------------------------------------------------------------------
!TS   use shr_kind_mod, only: r8 => shr_kind_r8
!TS   use zio,          only: zio_uf_read_double, zio_get_fileid, zio_noerr
!TS   use abortutils, only : endrun
!
! Arguments
!
!TS   integer, intent(in) :: filedesc   ! File descriptor
!TS   integer, intent(in) :: distarray  ! Dist array descriptor
!TS   real(r8) :: arr(*)                ! Array to be read
!
! Local workspace
!
!TS   integer ioerr, ioerr1      ! errorcodes
!TS   integer iu

!TS   ioerr = zio_uf_read_double (filedesc, distarray, arr)

!TS   if (ioerr /= ZIO_NOERR) then
!TS      ioerr1 = zio_get_fileid (filedesc,iu)
!TS      write (6,*) 'READIN_R8 ioerror ',ioerr,' on i/o unit = ',iu
!TS      call endrun('ZIO_ERROR')
!TS   end if

!TS   return
!TS   end subroutine pzreadin_r8

!#######################################################################

!TS   subroutine preadin_r4 (filedesc, arr, distarray)
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Wrapper routine to read binary file 
! 
! Author: 
! 
!-----------------------------------------------------------------------
!TS   use shr_kind_mod, only: r4 => shr_kind_r4
!TS   use zio,          only: zio_uf_read_real, zio_get_fileid, zio_noerr
!TS   use abortutils, only : endrun
!
! Arguments
!
!TS   integer, intent(in) :: filedesc   ! File descriptor
!TS   integer, intent(in) :: distarray  ! Dist array descriptor
!TS   real(r4) :: arr(*)                ! Array to be read
!
! Local workspace
!
!TS   integer ioerr, ioerr1      ! errorcodes
!TS   integer iu

!TS   ioerr = zio_uf_read_real (filedesc, distarray, arr)

!TS   if (ioerr /= ZIO_NOERR) then
!TS      ioerr1 = zio_get_fileid (filedesc,iu)
!TS      write (6,*) 'READIN_R4 ioerror ',ioerr,' on i/o unit = ',iu
!TS      call endrun('ZIO_ERROR')
!TS   end if

!TS   return
!TS   end subroutine preadin_r4

!#######################################################################

!TS   subroutine preadin_int (filedesc, arr, distarray)
!----------------------------------------------------------------------- 
! 
! Purpose: 
! Wrapper routine to read binary file 
! 
! Author: 
! 
!-----------------------------------------------------------------------
!TS   use zio,          only: zio_uf_read_int, zio_get_fileid, zio_noerr
!TS   use abortutils, only : endrun
!
! Arguments
!
!TS   integer, intent(in) :: filedesc   ! File descriptor
!TS   integer, intent(in) :: distarray  ! Dist array descriptor
!TS   integer :: arr(*)                 ! Array to be read
!
! Local workspace
!
!TS   integer ioerr, ioerr1      ! errorcodes
!TS   integer iu

!TS   ioerr = zio_uf_read_int (filedesc, distarray, arr)

!TS   if (ioerr /= ZIO_NOERR) then
!TS      ioerr1 = zio_get_fileid (filedesc,iu)
!TS      write (6,*) 'READIN_INT ioerror ',ioerr,' on i/o unit = ',iu
!TS      call endrun('ZIO_ERROR')
!TS   end if

!TS   return
!TS   end subroutine preadin_int
